@@ -18,24 +18,11 @@ summary.aftgee <- function(object,...){
   se.gee <- sqrt(diag(z$var.res))
   est.temp.gee <- ifelse(se.gee == "NaN", "NaN", est.gee)
   z.val.gee <- as.numeric(est.temp.gee)/as.numeric(se.gee)
-  TAB.gee <- cbind(Estimate = est.gee, StdErr = se.gee, z.value = z.val.gee, p.value = 2 * pnorm(-abs(z.val.gee)))
-  rownames(TAB.gee) <- rownames(z$coefficients)
-
-  ## initial part
-  if ( z$initial != "lm" & z$iniEst == FALSE & !(is.numeric(z$initial))) {
-
-
-      est.ini <- z$coef.init[which(is.na(z$sd.init) == FALSE)]
-      se.ini <- z$sd.init[which(is.na(z$sd.init) == FALSE)]
-      est.temp.ini <- ifelse(se.ini == "NaN", "NaN", est.ini)
-    z.val.ini <- as.numeric(est.temp.ini)/as.numeric(se.ini)
-    TAB.ini <- cbind(Estimate = est.ini, StdErr = se.ini, z.value = z.val.ini, p.value = 2 * pnorm(-abs(z.val.ini)))
-    rownames(TAB.ini) <- rownames(z$coefficients)[which(is.na(z$sd.init) == FALSE)]
-  }
-  inirow <- nrow(TAB.ini)
-  geerow <- nrow(TAB.gee)
-  TAB <- rbind(TAB.ini, TAB.gee)
-  res <- list(call=object$call, coefficients=TAB, initial = z$initial, iniEst = z$iniEst, est.ini = z$coef.init, inirow = inirow, geerow = geerow)
+  TAB <- cbind(Estimate = est.gee, StdErr = se.gee, z.value = z.val.gee, p.value = 2 * pnorm(-abs(z.val.gee)))
+  rownames(TAB) <- rownames(z$coefficients)
+  ## binit part
+  est.ini <- z$coef.init
+  res <- list(call=object$call, coefficients=TAB, binit = z$binit, iniEst = z$iniEst, est.ini = z$coef.init)
   class(res) <- "summary.aftgee"
   res
 }
@@ -70,31 +57,34 @@ print.summary.aftgee <- function(x, ...){
   cat("Call:\n")
   print(x$call)
   cat("\n")
-  if (x$iniEst == FALSE) {
-      if (x$initial != "lm") {
-          cat("Gehan Estimator:")
-          cat("\n")
-          printCoefmat(x$coefficients[1:x$inirow,], P.values = TRUE, has.Pvalue = TRUE)
-          cat("\n")
-          cat("AFTGEE Estimator")
-          cat("\n")
-          printCoefmat(x$coefficients[(x$inirow + 1):(x$inirow + x$geerow),], P.values = TRUE, has.Pvalue = TRUE)
-      }
-      else {
-          cat("AFTGEE Estimator")
-          cat("\n")
-          printCoefmat(x$coefficients, P.values = TRUE, has.Pvalue = TRUE)
-      }
-  }
-  if (x$iniEst == TRUE | is.numeric(x$initial)) {
-      cat("Gehan Initial Value:")
-      cat("\n")
-      cat(round(as.numeric(x$est.ini), digits = 7))
-      cat("\n")
-      cat("AFTGEE Estimator")
-      cat("\n")
-      printCoefmat(as.matrix(x$coefficients), P.values = TRUE, has.Pvalue = TRUE)
-  }
+  cat("AFTGEE Estimator")
+  cat("\n")
+  printCoefmat(as.matrix(x$coefficients), P.values = TRUE, has.Pvalue = TRUE)
+  ## if (is.numeric(x$binit) != TRUE) {
+  ##     if (x$binit == "lm") {
+  ##         cat("Initial Estimator from lm:")
+  ##         cat("\n")
+  ##         cat(format(round(as.numeric(x$est.ini), digits = 5), nsmall = 5))
+  ##     }
+  ##     if (x$binit == "srrgehan") {
+  ##         cat("Initial Estimator from smoothrr with Gehan's weight:")
+  ##         cat("\n")
+  ##         cat(format(round(as.numeric(x$est.ini), digits = 5), nsmall = 5))
+  ##     }
+  ##     cat("\n")
+  ##     cat("AFTGEE Estimator:")
+  ##     cat("\n")
+  ##     printCoefmat(x$coefficients, P.values = TRUE, has.Pvalue = TRUE)
+  ## }
+  ## if (is.numeric(x$binit) == TRUE) {
+  ##     cat("Gehan Estimator:")
+  ##     cat("\n")
+  ##     cat(format(round(as.numeric(x$est.ini), digits = 5), nsmall = 5))
+  ##     cat("\n")
+  ##     cat("AFTGEE Estimator")
+  ##     cat("\n")
+  ##     printCoefmat(as.matrix(x$coefficients), P.values = TRUE, has.Pvalue = TRUE)
+  ## }
 }
 
 
@@ -103,8 +93,6 @@ print.summary.smoothrr <- function(x, ...){
   cat("Call:\n")
   print(x$call)
   cat("\n")
-  cat("smoothrr estimator")
-  cat("\n")
   for (i in 1:se.count){
       cat("\n")
       cat("Variance Estimator:", as.character(x$var.name[i]))
